<?xml version="1.0"?>
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
               xmlns:f="http://www.data-structures.org/scalars/"
               xmlns:stream="http://www.data-structures.org/stream/" 
               xmlns:xs="http://www.w3.org/2001/XMLSchema"
               version="2.0">
 
    <!--
        Author: Roger L. Costello
        Date: September 11, 2010
        Version 1.0        
    -->
    
    <xsl:include href="scalar-functions.xsl"/>
    <xsl:include href="stream-functions.xsl"/>
    
     <xsl:output method="html"/>

    <xsl:template match="/">

        <html>
            <head>
                <title>Using Stream Functions</title>
            </head>
            <body>
                
                <h1>Using Stream Functions</h1>
                
                <!-- Process each bank transaction. -->
                
                <h2>Sum of the Bank Transactions</h2>
                <p>
                    <xsl:sequence select="
                        stream:iterator(
                            stream:accumulate(
                                $credit-debit, 
                                100, 
                                stream:enumerate-following-siblings(/transactions/child::*[1])
                            )
                         )" />
                </p>
                
                
                <!-- What is the second prime number in the interval from 10,000 to 100,000,000? -->
                
                <h2>Second Prime Number in the Interval 10,000 to 100,000,000</h2>
                <p>
                    <xsl:sequence select="
                        stream:first(
                            stream:rest(
                                stream:rest(
                                    stream:filter(
                                        $prime, 
                                        stream:enumerate-interval(10000, 100000000)
                                    )
                                )
                            )
                        )" />
                </p>
                
                
                <!-- Generate the odd values in the range 1 to 100: -->
                
                <h2>Odd Values in the Range 1 to 100</h2>
                <p>
                    <xsl:sequence select="
                        stream:iterator(
                            stream:filter($odd, (stream:enumerate-interval(1, 100)))
                        )" />
                </p>
                
                
                <!-- Apply (i.e., map) the absolute value function to the integers in the interval -5 to 5: -->
                
                <h2>Absolute Value of the Integers in the Interval -5 to 5</h2>
                <p>
                    <xsl:sequence select="
                        stream:iterator(
                            stream:map($stream-abs, stream:enumerate-interval(-5, 5))
                        )" />
                </p>
                
                
                <!-- What is the hundredth prime number in the range 1 to 1,000,000: -->
                
                <h2>The 100th Prime Number in the Range 1 to 1,000,000</h2>
                <p>
                    <xsl:sequence select="
                        stream:ref(
                            stream:filter(
                                $prime, 
                                stream:enumerate-interval(1, 1000000)
                            ), 
                            100
                        )" />
                </p>
                
              </body>
        </html>


    </xsl:template>

</xsl:transform>
