<?xml version="1.0"?>
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
               xmlns:f="http://www.data-structures.org/scalars/" 
               xmlns:xs="http://www.w3.org/2001/XMLSchema"
               version="2.0">
 
    <!--
        Author: Roger L. Costello
        Date: September 11, 2010 
        Version 1.0       
    -->   
    
    <xsl:variable name="prime" select="document('')//f:prime" />
    
    <f:prime/>
    
    <xsl:template match="f:prime" as="xs:boolean">
        <xsl:param name="n" as="xs:integer"/>
        
        <xsl:sequence select="f:prime($n)" />
        
    </xsl:template>
    
    <xsl:function name="f:prime" as="xs:boolean">
        <xsl:param name="n" as="xs:integer" />
        
        <xsl:sequence select="f:smallest-divisor($n) eq $n" />
        
    </xsl:function>
    
    <xsl:function name="f:sum-primes" as="xs:integer">
        <xsl:param name="a" as="xs:integer" />
        <xsl:param name="b" as="xs:integer" />
        
        <xsl:sequence select="f:iter($b, $a, 0)" />
        
    </xsl:function>
    
    <xsl:function name="f:iter" as="xs:integer">
        <xsl:param name="b" as="xs:integer" />
        <xsl:param name="count" as="xs:integer" />
        <xsl:param name="accum" as="xs:integer" />
        
        <xsl:choose>
            <xsl:when test="$count gt $b">
                <xsl:sequence select="$accum" />
            </xsl:when>
            <xsl:when test="f:prime($count)">
                <xsl:sequence select="f:iter($b, $count + 1, $accum + $count)" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="f:iter($b, $count + 1, $accum)" />
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:function>
    

    <xsl:function name="f:smallest-divisor" as="xs:integer">
        <xsl:param name="n" as="xs:integer" />

        <xsl:sequence select="f:find-divisor($n, 2)" />
        
    </xsl:function>

    <xsl:function name="f:find-divisor" as="xs:integer">
        <xsl:param name="n" as="xs:integer" />
        <xsl:param name="test-divisor" as="xs:integer" />

        <xsl:choose>
            <xsl:when test="f:square($test-divisor) gt $n">
                <xsl:sequence select="$n" />
            </xsl:when>
            <xsl:when test="f:divides($n, $test-divisor)">
                <xsl:sequence select="$test-divisor" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="f:find-divisor($n, $test-divisor  +1)" />
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:function>
    
    <xsl:function name="f:square" as="xs:integer">
        <xsl:param name="n" as="xs:integer" />
        
        <xsl:sequence select="$n * $n" />
        
    </xsl:function>
    
    <xsl:function name="f:divides" as="xs:boolean">
        <xsl:param name="a" as="xs:integer" />
        <xsl:param name="b" as="xs:integer" />
        
        <xsl:sequence select="($a mod $b) eq 0" />
        
    </xsl:function>
    

    <xsl:variable name="odd" select="document('')//f:odd" />
    
    <f:odd/>
    
    <xsl:template match="f:odd">
        <xsl:param name="n" />
        
        <xsl:sequence select="f:odd($n)" />
        
    </xsl:template>
    
    <xsl:function name="f:odd" as="xs:boolean*">
        <xsl:param name="n" as="xs:integer*" />
        
        <xsl:sequence select="not(f:divides($n, 2))" />
        
    </xsl:function>
    
    <xsl:variable name="add" select="document('')//f:add" />
    
    <f:add/>
    
    <xsl:template match="f:add" as="xs:integer">
        <xsl:param name="opnd1" as="xs:integer" />
        <xsl:param name="opnd2" as="xs:integer" />
        
        <xsl:sequence select="f:add($opnd1, $opnd2)" />
        
    </xsl:template>
    
    <xsl:function name="f:add" as="xs:integer">
        <xsl:param name="opnd1" as="xs:integer" />
        <xsl:param name="opnd2" as="xs:integer" />
        
        <xsl:sequence select="$opnd1 + $opnd2" />
        
    </xsl:function>
    
    <xsl:variable name="multiply" select="document('')//f:multiply" />
    
    <f:multiply/>
    
    <xsl:template match="f:multiply" as="xs:integer">
        <xsl:param name="opnd1" as="xs:integer" />
        <xsl:param name="opnd2" as="xs:integer" />
        
        <xsl:sequence select="f:multiply($opnd1, $opnd2)" />
        
    </xsl:template>
    
    <xsl:function name="f:multiply" as="xs:integer">
        <xsl:param name="opnd1" as="xs:integer" />
        <xsl:param name="opnd2" as="xs:integer" />
        
        <xsl:sequence select="$opnd1 * $opnd2" />
        
    </xsl:function>
    
    <xsl:variable name="cube" select="document('')//f:cube" />
    
    <f:cube/>
    
    <xsl:template match="f:cube" as="xs:integer">
        <xsl:param name="n" as="xs:integer" />
        
        <xsl:sequence select="f:cube($n)" />
        
    </xsl:template>
    
    <xsl:function name="f:cube" as="xs:integer">
        <xsl:param name="n" as="xs:integer" />
        
        <xsl:sequence select="$n * $n * $n" />
        
    </xsl:function>
    
    <xsl:variable name="inc" select="document('')//f:inc" />
    
    <f:inc/>
    
    <xsl:template match="f:inc" as="xs:integer">
        <xsl:param name="n" as="xs:integer" />
        
        <xsl:sequence select="f:inc($n)" />
        
    </xsl:template>
    
    <xsl:function name="f:inc" as="xs:integer">
        <xsl:param name="n" as="xs:integer" />
        
        <xsl:sequence select="$n + 1" />
        
    </xsl:function>
    
    <xsl:variable name="sum-cubes" select="document('')//f:sum-cubes" />
    
    <f:sum-cubes/>
    
    <xsl:template match="f:sum-cubes" as="xs:integer">
        <xsl:param name="a" as="xs:integer" />
        <xsl:param name="b" as="xs:integer" />
        
        <xsl:sequence select="f:sum-cubes($a, $b)" />
        
    </xsl:template>
    
    <xsl:function name="f:sum-cubes" as="xs:integer">
        <xsl:param name="a" as="xs:integer" />
        <xsl:param name="b" as="xs:integer" />
        
        <xsl:sequence select="f:sum($cube, $a, $inc, $b)" />
        
    </xsl:function>
    
    <xsl:variable name="sum" select="document('')//f:sum" />
    
    <f:sum/>
    
    <xsl:template match="f:sum" as="xs:integer">
        <xsl:param name="term" as="element()" />
        <xsl:param name="a" as="xs:integer" />
        <xsl:param name="next" as="element()" />
        <xsl:param name="b" as="xs:integer" />
        
        <xsl:sequence select="f:sum($term, $a, $next, $b)" />
        
    </xsl:template>
    
    <xsl:function name="f:sum" as="xs:integer">
        <xsl:param name="term" as="element()" />
        <xsl:param name="a" as="xs:integer" />
        <xsl:param name="next" as="element()" />
        <xsl:param name="b" as="xs:integer" />
        
        <xsl:choose>
            <xsl:when test="$a gt $b">
                <xsl:sequence select="0" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="term_a">
                    <xsl:apply-templates select="$term">
                        <xsl:with-param name="n" select="$a"/>
                    </xsl:apply-templates>
                </xsl:variable>
                <xsl:variable name="next_a">
                    <xsl:apply-templates select="$next">
                        <xsl:with-param name="n" select="$a" />
                    </xsl:apply-templates>
                </xsl:variable>
                <xsl:sequence select="xs:integer($term_a + f:sum($term, $next_a, $next, $b))" />
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:function>
    
    <xsl:variable name="credit-debit" select="document('')//f:credit-debit" />
    
    <f:credit-debit/>
    
    <xsl:template match="f:credit-debit" as="item()*">
        <xsl:param name="opnd1" as="xs:integer" />
        <xsl:param name="opnd2" as="element()" />
        
        <xsl:variable name="balance" select="$opnd1" />
        <xsl:variable name="transaction" select="$opnd2" />
                
        <xsl:sequence select="f:credit-debit($balance, $transaction)" />
        
    </xsl:template>
    
    <xsl:function name="f:credit-debit" as="xs:integer">
        <xsl:param name="balance" as="xs:integer" />
        <xsl:param name="transaction" as="element()" />
        
        <xsl:choose>
            <xsl:when test="$transaction[self::deposit]">
                <xsl:sequence select="$balance + xs:integer($transaction/text())" />
            </xsl:when>
            <xsl:when test="$transaction[self::withdrawal]">
                <xsl:sequence select="$balance - xs:integer($transaction/text())" />
            </xsl:when>
            <xsl:otherwise>
                <xsl:sequence select="$balance" />
            </xsl:otherwise>
        </xsl:choose>
        
    </xsl:function>

</xsl:transform>